==> mod_trackback-0.1.6.tar.gz: mod_trackback-0.1.6/mod_trackback.c <==
/* Copyright (C) 2004 Hiroyuki OYAMA

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "httpd.h"
#include "ap_config.h"
#include "http_config.h"
#include "http_log.h"
#include "http_protocol.h"
#include "http_main.h"
#include "http_core.h"

#include "trackback.h"
#include "encoding.h"
#include "storage.h"
#include "util.h"


static void *create_dir_conf(pool *p, char *path)
{
    trackback_conf *conf;

    conf = ap_pcalloc(p, sizeof(trackback_conf));
    conf->storage = NULL;

    return (void *)conf;
}



static int recv_trackback_ping(request_rec *r)
{
    const char *title, *excerpt, *url, *blog_name, *org_charset, *charset;
    const char *body;
    const char *msg = NULL;
    int rc = 0;
    iconv_t cd;
    table *param;
    mtb_trackback_message_rec *message;

    param = (table *)ap_table_get(r->notes, REQUEST_PARAM_FIELD);
    if (param == NULL) {
        ap_log_rerror(APLOG_MARK, APLOG_ERR|APLOG_NOERRNO, r,
                      "cannot find \"%s\" field in r->notes", REQUEST_PARAM_FIELD);
        return HTTP_INTERNAL_SERVER_ERROR;
    }

    title     = mtb_strip_html_tag(r->pool, ap_table_get(param, "title"));
    excerpt   = mtb_strip_html_tag(r->pool, ap_table_get(param, "excerpt"));
    url       = mtb_strip_html_tag(r->pool, ap_table_get(param, "url"));
    blog_name = mtb_strip_html_tag(r->pool, ap_table_get(param, "blog_name"));
    org_charset   = ap_table_get(param, "charset");
    charset = org_charset;

    /* require url param */
    if (url == NULL || *url == '\0'
        || (strncmp(url, "http://", sizeof("http://")-1) != 0
            && strncmp(url, "https://", sizeof("https://")-1) != 0))
    {
        rc = 1;
        msg = "require url";
        msg = ap_psprintf(r->pool, TRACKBACK_R_MESSAGE, msg);
        body = ap_psprintf(r->pool, TRACKBACK_RESPONSE, rc, msg);
        r->content_type = "text/xml; charset=iso-8859-1";
        r->mtime = 0;
        ap_send_http_header(r);
        ap_rputs(body, r);
        return OK;
    }

    if (charset == NULL) {
        charset = "UTF-8";
    }
    charset = mtb_find_encoding(charset);
    if (charset == NULL)
        charset = "UTF-8";
    if ((cd = mtb_iconv_open(r->pool, charset, "UTF-8")) == (iconv_t)(-1)) {
        /*
        ap_log_rerror(APLOG_MARK, APLOG_WARNING|APLOG_NOERRNO, r,
                      "cannot open iconv conversion descriptor: from=%s to=%s: %s",
                      charset, "UTF-8", r->uri);
        */
    }

    if (title == NULL || *title == '\0')
        title = url;
    else
        title = mtb_convert_encoding(r->pool, cd, title);

    if (excerpt == NULL || *excerpt == '\0')
        excerpt = "";
    else
        excerpt = mtb_convert_encoding(r->pool, cd, excerpt);

    if (blog_name == NULL || *blog_name == '\0')
        blog_name = url;
    else
        blog_name = mtb_convert_encoding(r->pool, cd, blog_name);
    mtb_iconv_close(cd);

    message = ap_palloc(r->pool, sizeof(mtb_trackback_message_rec));
    message->id = ap_pstrdup(r->pool, ap_table_get(param, "id"));
    message->title = title;
    message->excerpt = excerpt;
    message->blog_name = blog_name;
    message->url = url;
    message->date = r->request_time;
    if (mtb_store_message(r, message) != OK) {
        msg = "cannot store trackback message: undefined TrackBack ID or Storage error";
        rc = 1;
    }


    /* send response */
    if (rc == 0) {
        msg = "Ping saved successfully.";
    }
    msg = ap_psprintf(r->pool, TRACKBACK_R_MESSAGE, msg ? msg : "");
    body = ap_psprintf(r->pool, TRACKBACK_RESPONSE, rc, msg);

    r->content_type = "text/xml; charset=iso-8859-1";
    r->mtime = 0;
    ap_send_http_header(r);
    ap_rputs(body, r);

    return OK;
}




static int send_trackbacks_rss(request_rec *r)
{
    mtb_trackback_message_rec *msg = NULL;
    const char *id, *org_encoding, *encoding;
    char *body;
    table *param;
    struct tm *tms;
    unsigned char timestr[HUGE_STRING_LEN];
    iconv_t cd;

    param = (table *)ap_table_get(r->notes, REQUEST_PARAM_FIELD);
    org_encoding = ap_table_get(param, "charset");
    encoding = org_encoding;
    if (encoding == NULL) {
        encoding = "UTF-8";
        org_encoding = encoding;
    }
    encoding = mtb_find_encoding(encoding);
    if (encoding == NULL) {
        encoding = "UTF-8";
        org_encoding = "UTF-8";
    }

    if ((cd = mtb_iconv_open(r->pool, "UTF-8", encoding)) == (iconv_t)(-1)) {
        /*
        ap_log_rerror(APLOG_MARK, APLOG_WARNING|APLOG_NOERRNO, r,
                      "cannot open iconv conversion descriptor: from=%s to=%s: %s",
                      "UTF-8", encoding, r->uri);
        */
    }


    id = ap_table_get(param, "id");
    msg = mtb_load_message(r, id);

    r->content_type = "application/rdf+xml";
    ap_send_http_header(r);

    ap_rprintf(r,
               "<?xml version=\"1.0\" encoding=\"%s\"?>\n"
               "<response>\n"
               "  <error>0</error>\n"
               "  <rss version=\"0.91\">\n"
               "    <channel>\n"
               "      <title>TrackBack Server</title>\n"
               "      <link>%s</link>\n"
               "      <description>Generated by mod_trackback.c</description>\n",
               org_encoding,
               ap_construct_url(r->pool,
                                ap_pstrcat(r->pool, r->uri, "?",
                                           ap_escape_uri(r->pool, r->args),
                                           NULL),
                                r));
    for (; msg != NULL; msg = msg->next) {
        tms = localtime(&(msg->date));
        strftime(timestr, sizeof(timestr), "%Y/%m/%d %H:%M:%S", tms);
        timestr[sizeof(timestr) - 1] = '\0';

        body = ap_psprintf(r->pool,
                           "      <item>\n"
                           "        <title>%s</title>\n"
                           "        <link>%s</link>\n"
                           "        <description>%s</description>\n"
                           "      </item>\n",
                           msg->title, msg->url,
                           msg->excerpt);
        body = (char *)mtb_convert_encoding(r->pool, cd, body);
        ap_rputs(body ? body : "", r);
    }
    ap_rputs("    </channel>\n  </rss>\n</response>\n", r);
    mtb_iconv_close(cd);

    return OK;
}

static int send_trackbacks_javascript(request_rec *r)
{
    mtb_trackback_message_rec *msg = NULL;
    mtb_trackback_message_rec *summsg = NULL;
    int msg_count = 0;
    const char *id, *org_encoding, *encoding;
    char *body;
    table *param;
    struct tm *tms;
    unsigned char timestr[HUGE_STRING_LEN];
    iconv_t cd;

    param = (table *)ap_table_get(r->notes, REQUEST_PARAM_FIELD);
    org_encoding = ap_table_get(param, "charset");
    encoding = org_encoding;
    if (encoding == NULL) {
        encoding = "UTF-8";
        org_encoding = "UTF-8";
    }
    encoding = mtb_find_encoding(encoding);
    if (encoding == NULL) {
        encoding = "UTF-8";
        org_encoding = "UTF-8";
    }

    if ((cd = mtb_iconv_open(r->pool, "UTF-8", encoding)) == (iconv_t)(-1)) {
        /*
        ap_log_rerror(APLOG_MARK, APLOG_WARNING|APLOG_NOERRNO, r,
                      "cannot open iconv conversion descriptor: from=%s to=%s: %s",
                      "UTF-8", encoding, r->uri);
        */
    }

    id = ap_table_get(param, "id");
    msg = mtb_load_message(r, id);

    r->content_type = "application/x-javascript";
    ap_send_http_header(r);

    for (summsg = msg; summsg != NULL; summsg = summsg->next)
        msg_count++;
    ap_rprintf(r,
               "document.writeln('<div class=\"mtb_body\">');\n"
               "document.writeln('  <div class=\"mtb_summary\"><a href=\"javascript:flip_tb(\\'mtb_entry\\')\">TrackBack(%d)</a></div>');\n"
               "document.writeln('  <div class=\"mtb_trackback_url\">%s</div>');\n",
               msg_count,
               ap_construct_url(r->pool, r->uri, r));
    for (; msg != NULL; msg = msg->next) {
        tms = localtime(&(msg->date));
        strftime(timestr, sizeof(timestr), "%Y/%m/%d %H:%M:%S", tms);
        timestr[sizeof(timestr) - 1] = '\0';

        body = ap_psprintf(r->pool,
                   "document.writeln('  <div class=\"mtb_entry\">');\n"
                   "document.writeln('    <div class=\"mtb_title\"><a href=\"%s\" class=\"mtb_title\">%s</a></div>');\n"
                   "document.writeln('    <div class=\"mtb_excerpt\">%s</div>');\n"
                   "document.writeln('    <div class=\"mtb_blogname\">%s</div>');\n"
                   "document.writeln('    <div class=\"mtb_date\">%s</div>');\n"
                   "document.writeln('  </div>');\n",
                   msg->url, msg->title,
                   mtb_escape_html(r->pool, msg->excerpt),
                   msg->blog_name,
                   timestr);
        body = (char *)mtb_convert_encoding(r->pool, cd, body);
        ap_rputs(body ? body : "", r);
    }
    ap_rputs("document.writeln('</div>');\n"
             "function flip_tb(class_name) {\n"
             "    var nodes = document.getElementsByTagName('div');\n"
             "    for (var i = 0; i < nodes.length; i++) {\n"
             "        if (nodes.item(i).className != class_name) {\n"
             "            continue;\n"
             "        }\n"
             "        var node = nodes.item(i);\n"
             "        node.style.display = node.style.display == 'none'\n"
             "                             ? 'block' : 'none';\n"
             "    }\n"
             "}\n"
             "flip_tb('mtb_entry');\n",
             r);
    mtb_iconv_close(cd);

    return OK;
}

static int send_trackbacks_html(request_rec *r)
{
    mtb_trackback_message_rec *msg = NULL;
    const char *id, *org_encoding, *encoding;
    char *body;
    table *param;
    struct tm *tms;
    unsigned char timestr[HUGE_STRING_LEN];
    iconv_t cd;

    param = (table *)ap_table_get(r->notes, REQUEST_PARAM_FIELD);
    org_encoding = ap_table_get(param, "charset");
    encoding = org_encoding;
    if (encoding == NULL) {
        encoding = "UTF-8";
        org_encoding = "UTF-8";
    }
    encoding = mtb_find_encoding(encoding);
    if (encoding == NULL) {
        encoding = "UTF-8";
        org_encoding = "UTF-8";
    }
    if ((cd = mtb_iconv_open(r->pool, "UTF-8", encoding)) == (iconv_t)(-1)) {
        /*
        ap_log_rerror(APLOG_MARK, APLOG_WARNING|APLOG_NOERRNO, r,
                      "cannot open iconv conversion descriptor: from=%s to=%s: %s",
                      "UTF-8", encoding, r->uri);
        */
    }

    id = ap_table_get(param, "id");
    msg = mtb_load_message(r, id);

    r->content_type = "text/html";
    ap_send_http_header(r);

    ap_rprintf(r,
               "<html>\n"
               "  <head>\n"
               "    <meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\" />\n"
               "  </head>\n"
               "  <body>\n"
               "    <div class=\"mtb_body\">\n"
               "      <div class=\"mtb_trackback_url\">%s</div>\n",
               org_encoding,
               ap_construct_url(r->pool, r->uri, r));
    for (; msg != NULL; msg = msg->next) {
        tms = localtime(&(msg->date));
        strftime(timestr, sizeof(timestr), "%Y/%m/%d %H:%M:%S", tms);
        timestr[sizeof(timestr) - 1] = '\0';

        body = ap_psprintf(r->pool,
                   "      <div class=\"mtb_entry\">\n"
                   "        <div class=\"mtb_title\"><a href=\"%s\" class=\"mtb_title\">%s</a></div>\n"
                   "        <div class=\"mtb_excerpt\">%s</div>\n"
                   "        <div class=\"mtb_blogname\">%s</div>\n"
                   "        <div class=\"mtb_date\">%s</div>\n"
                   "      </div>\n",
                   msg->url, msg->title,
                   msg->excerpt,
                   msg->blog_name,
                   timestr);
        body = (char *)mtb_convert_encoding(r->pool, cd, body);
        ap_rputs(body ? body : "", r);
    }
    ap_rputs("    </div>\n</body>\n</html>\n", r);
    mtb_iconv_close(cd);

    return OK;
}

static int trackback_handler(request_rec *r)
{
    table *param;
    const char *mode, *id;

    param = mtb_parse_request(r);
    ap_table_setn(r->notes, REQUEST_PARAM_FIELD, (const char *)param);
    mode = ap_table_get(param, "__mode");
    id   = ap_table_get(param, "id");
    if (id == NULL || *id == '\0') {
        return HTTP_BAD_REQUEST;
    }
    if (mode == NULL)
        mode = "html";  /* set to default mode */

    if (r->method_number == M_POST) {
        return recv_trackback_ping(r);
    }
    else if (r->method_number == M_GET) {
        if (strcmp(mode, "rss") == 0) {
            return send_trackbacks_rss(r);
        }
        else if (strcmp(mode, "javascript") == 0 || strcmp(mode, "js") == 0) {
            return send_trackbacks_javascript(r);
        }
        else {
            return send_trackbacks_html(r);
        }
    }
    else {
        return HTTP_NOT_IMPLEMENTED;
    }
}

/* Dispatch list of content handlers */
static const handler_rec trackback_handlers[] = { 
    { "trackback", trackback_handler }, 
    { NULL, NULL }
};


static const command_rec commands[]= {
    { "TrackbackStorage", ap_set_file_slot,
      (void *)XtOffsetOf(trackback_conf, storage), ACCESS_CONF|OR_NONE, TAKE1,
      "trackback database path" },
    { NULL }
};

/* Dispatch list for API hooks */
module MODULE_VAR_EXPORT trackback_module = {
    STANDARD_MODULE_STUFF, 
    NULL,                  /* module initializer                  */
    create_dir_conf,       /* create per-dir    config structures */
    NULL,                  /* merge  per-dir    config structures */
    NULL,                  /* create per-server config structures */
    NULL,                  /* merge  per-server config structures */
    commands,              /* table of config file commands       */
    trackback_handlers,    /* [#8] MIME-typed-dispatched handlers */
    NULL,                  /* [#1] URI to filename translation    */
    NULL,                  /* [#4] validate user id from request  */
    NULL,                  /* [#5] check if the user is ok _here_ */
    NULL,                  /* [#3] check access by host address   */
    NULL,                  /* [#6] determine MIME type            */
    NULL,                  /* [#7] pre-run fixups                 */
    NULL,                  /* [#9] log a transaction              */
    NULL,                  /* [#2] header parser                  */
    NULL,                  /* child_init                          */
    NULL,                  /* child_exit                          */
    NULL                   /* [#0] post read-request              */
#ifdef EAPI
   ,NULL,
    NULL,
    NULL,
    NULL
#endif
};

==> mod_trackback-1.0.1.tar.gz: mod_trackback-1.0.1/mod_trackback.c <==
/* Copyright (C) 2004 Hiroyuki OYAMA

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "httpd.h"
#include "ap_config.h"
#include "http_config.h"
#include "http_log.h"
#include "http_protocol.h"
#include "http_main.h"
#include "http_core.h"

#include "trackback.h"
#include "encoding.h"
#include "util.h"


static void *create_dir_conf(pool *p, char *path)
{
    trackback_conf *conf;

    conf = ap_pcalloc(p, sizeof(trackback_conf));
    conf->storage = NULL;

    return (void *)conf;
}



static int recv_trackback_ping(request_rec *r)
{
    const char *title, *excerpt, *url, *blog_name, *org_charset, *charset;
    const char *body;
    const char *msg = NULL;
    int rc = 0;
    iconv_t cd;
    table *param;
    mtb_message_rec *message;

    param = (table *)ap_table_get(r->notes, REQUEST_PARAM_FIELD);
    if (param == NULL) {
        ap_log_rerror(APLOG_MARK, APLOG_ERR|APLOG_NOERRNO, r,
                      "cannot find \"%s\" field in r->notes", REQUEST_PARAM_FIELD);
        return HTTP_INTERNAL_SERVER_ERROR;
    }

    title     = mtb_strip_html_tag(r->pool, ap_table_get(param, "title"));
    excerpt   = mtb_strip_html_tag(r->pool, ap_table_get(param, "excerpt"));
    url       = mtb_strip_html_tag(r->pool, ap_table_get(param, "url"));
    blog_name = mtb_strip_html_tag(r->pool, ap_table_get(param, "blog_name"));
    org_charset   = ap_table_get(param, "charset");
    charset = org_charset;
    if (charset == NULL) {
        charset = "UTF-8";
    }
    charset = mtb_find_encoding(charset);
    if ((cd = mtb_iconv_open(r->pool, charset, "UTF-8")) == (iconv_t)(-1)) {
        /*
        ap_log_rerror(APLOG_MARK, APLOG_WARNING|APLOG_NOERRNO, r,
                      "cannot open iconv conversion descriptor: from=%s to=%s: %s",
                      charset, "UTF-8", r->uri);
        */
    }

    if (url == NULL)
        url = "";

    if (title == NULL || *title == '\0')
        title = url;
    else
        title = mtb_convert_encoding(r->pool, cd, title);

    if (excerpt == NULL || *excerpt == '\0')
        excerpt = "";
    else
        excerpt = mtb_convert_encoding(r->pool, cd, excerpt);

    if (blog_name == NULL || *blog_name == '\0')
        blog_name = url;
    else
        blog_name = mtb_convert_encoding(r->pool, cd, blog_name);
    mtb_iconv_close(cd);

    message = ap_palloc(r->pool, sizeof(mtb_message_rec));
    message->id = ap_pstrdup(r->pool, ap_table_get(param, "id"));
    message->title = title;
    message->excerpt = excerpt;
    message->blog_name = blog_name;
    message->url = url;
    message->date = r->request_time;
    message->hidden = 0;
    rc = mtb_validate_ping(r, message);
    if (rc != OK) {
        msg = "cannot accept trackback message";
        rc = 2;
    }
    else if (mtb_store_message(r, message) != OK) {
        msg = "cannot store trackback message: undefined TrackBack ID or Storage error";
        rc = 1;
    }


    /* send response */
    if (rc == 0) {
        msg = "Ping saved successfully.";
    }
    msg = ap_psprintf(r->pool, TRACKBACK_R_MESSAGE, msg ? msg : "");
    body = ap_psprintf(r->pool, TRACKBACK_RESPONSE, rc, msg);

    r->content_type = "text/xml; charset=iso-8859-1";
    r->mtime = 0;
    ap_send_http_header(r);
    ap_rputs(body, r);

    return OK;
}


static int send_contents(request_rec *r)
{
    mtb_message_rec *msg = NULL;
    table *param;
    int rc;
    const char *id;

    param = (table *)ap_table_get(r->notes, REQUEST_PARAM_FIELD);
    id = ap_table_get(param, "id");
    msg = mtb_load_message(r, id);
    rc = mtb_send_contents(r, msg);

    return rc;
}


static int trackback_handler(request_rec *r)
{
    table *param;
    const char *mode, *id;

    param = mtb_parse_request(r);
    ap_table_setn(r->notes, REQUEST_PARAM_FIELD, (const char *)param);
    mode = ap_table_get(param, "__mode");
    id   = ap_table_get(param, "id");
    if (id == NULL || *id == '\0') {
        return HTTP_BAD_REQUEST;
    }
    if (mode == NULL) {
        mode = "html";
        ap_table_set(param, "__mode", mode);  /* set to default mode */
    }

    if (r->method_number == M_POST) {
        return recv_trackback_ping(r);
    }
    else if (r->method_number == M_GET) {
        return send_contents(r);
    }
    else {
        return HTTP_NOT_IMPLEMENTED;
    }
}

/* Dispatch list of content handlers */
static const handler_rec trackback_handlers[] = { 
    { "trackback", trackback_handler }, 
    { NULL, NULL }
};


static const command_rec commands[]= {
    { "TrackbackStorage", ap_set_file_slot,
      (void *)XtOffsetOf(trackback_conf, storage), ACCESS_CONF|OR_NONE, TAKE1,
      "trackback database path" },
    { NULL }
};

/* Dispatch list for API hooks */
module MODULE_VAR_EXPORT trackback_module = {
    STANDARD_MODULE_STUFF, 
    NULL,                  /* module initializer                  */
    create_dir_conf,       /* create per-dir    config structures */
    NULL,                  /* merge  per-dir    config structures */
    NULL,                  /* create per-server config structures */
    NULL,                  /* merge  per-server config structures */
    commands,              /* table of config file commands       */
    trackback_handlers,    /* [#8] MIME-typed-dispatched handlers */
    NULL,                  /* [#1] URI to filename translation    */
    NULL,                  /* [#4] validate user id from request  */
    NULL,                  /* [#5] check if the user is ok _here_ */
    NULL,                  /* [#3] check access by host address   */
    NULL,                  /* [#6] determine MIME type            */
    NULL,                  /* [#7] pre-run fixups                 */
    NULL,                  /* [#9] log a transaction              */
    NULL,                  /* [#2] header parser                  */
    NULL,                  /* child_init                          */
    NULL,                  /* child_exit                          */
    NULL                   /* [#0] post read-request              */
#ifdef EAPI
   ,NULL,
    NULL,
    NULL,
    NULL
#endif
};
